#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDebug>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    setWindowFlags(Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);

    settings = new QSettings("mck182", "kstopwatch");

    seconds = settings->value("kstopwatch/seconds", 0).toUInt();
    alwaysOnTop = true;

    ui->setupUi(this);
    createActions();
    createTrayIcon();

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(updateTime()));
    timer->start(1000);

    if(settings->contains("kstopwatch/shortcut")) {
        slotKeySequenceChanged(QKeySequence::fromString(settings->value("kstopwatch/shortcut").toString()));
    }
}

MainWindow::~MainWindow()
{
    togglePauseWithKeyboardAction->forgetGlobalShortcut();
    settings->setValue("kstopwatch/seconds", seconds);
    delete settings;
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::updateTime() {
    seconds++;
    int minutes = seconds / 60;
    int hours = minutes / 60;
    int displayedSeconds = seconds % 60;
    QString secondsString = (displayedSeconds < 10) ? "0" + QString::number(displayedSeconds) : QString::number(displayedSeconds);
    QString minutesString = (minutes < 10) ? "0" + QString::number(minutes) : QString::number(minutes);
    QString hoursString = (hours < 10) ? "0" + QString::number(hours) : QString::number(hours);
    ui->label->setText(hoursString + ":" + minutesString + ":" + secondsString);
}

void MainWindow::createActions() {
    hideAction = new QAction(tr("Hide"), this);
    connect(hideAction, SIGNAL(triggered()), this, SLOT(slotHide()));

    showAction = new QAction(tr("Show"), this);
    connect(showAction, SIGNAL(triggered()), this, SLOT(slotShow()));

    pauseAction = new QAction(tr("Pause"), this);
    connect(pauseAction, SIGNAL(triggered()), this, SLOT(slotPause()));

    resetAction = new QAction(tr("Reset"), this);
    connect(resetAction, SIGNAL(triggered()), this, SLOT(slotReset()));

    resumeAction = new QAction(tr("Resume"), this);
    connect(resumeAction, SIGNAL(triggered()), this, SLOT(slotResume()));

    disableAOTAction = new QAction(tr("Disable always on top"), this);
    connect(disableAOTAction, SIGNAL(triggered()), this, SLOT(slotDisableAOT()));

    openOptionsDialogAction = new QAction(tr("Options..."), this);
    connect(openOptionsDialogAction, SIGNAL(triggered()), this, SLOT(slotOptions()));

    quitAction = new QAction(tr("&Quit"), this);
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

    togglePauseWithKeyboardAction = new KAction(tr("Toggle pause"), this);
    connect(togglePauseWithKeyboardAction, SIGNAL(triggered()), this, SLOT(slotTogglePause()));
    togglePauseWithKeyboardAction->setObjectName("toggleStopwatchPause");
    togglePauseWithKeyboardAction->forgetGlobalShortcut();
}

void MainWindow::createTrayIcon()
{
    trayIconMenu = new QMenu(this);
    trayIconMenu->addAction(pauseAction);
    trayIconMenu->addAction(resetAction);
    trayIconMenu->addAction(hideAction);
    trayIconMenu->addAction(disableAOTAction);
    trayIconMenu->addAction(openOptionsDialogAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(quitAction);

    QIcon icon(QPixmap(QString::fromUtf8(":/icons/systray")));
    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setContextMenu(trayIconMenu);
    trayIcon->setIcon(icon);
    trayIcon->show();
}

void MainWindow::slotDisableAOT() {
    if(alwaysOnTop) {
        setWindowFlags(Qt::Window | Qt::FramelessWindowHint);
        alwaysOnTop = false;
        disableAOTAction->setText(tr("Enable always on top"));
    }
    else {
        setWindowFlags(Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
        alwaysOnTop = true;
        disableAOTAction->setText(tr("Disable always on top"));
    }
    this->show();
}

void MainWindow::slotHide() {
    trayIconMenu->insertAction(hideAction, showAction);
    trayIconMenu->removeAction(hideAction);
    hide();
}

void MainWindow::slotShow() {
    trayIconMenu->insertAction(showAction, hideAction);
    trayIconMenu->removeAction(showAction);
    show();
}

void MainWindow::slotPause() {
    trayIconMenu->insertAction(pauseAction, resumeAction);
    trayIconMenu->removeAction(pauseAction);

    timer->stop();
}

void MainWindow::slotResume() {
    trayIconMenu->insertAction(resumeAction, pauseAction);
    trayIconMenu->removeAction(resumeAction);

    timer->start(1000);
}

void MainWindow::slotReset() {
    seconds = 0;
}

void MainWindow::slotOptions() {
    m_optionsDialog = new optionsDialog(this);
    m_optionsDialog->show();
    connect(m_optionsDialog, SIGNAL(newKeySequence(QKeySequence)), this, SLOT(slotKeySequenceChanged(QKeySequence)));
}

void MainWindow::slotKeySequenceChanged(QKeySequence seq) {
    qDebug() << "Setting the shortcut to " << seq.toString();
    //pauseAction->setShortcut(seq);
    //pauseAction->setShortcutContext(Qt::ApplicationShortcut);

    togglePauseWithKeyboardAction->setGlobalShortcut(KShortcut(seq), KAction::ActiveShortcut | KAction::DefaultShortcut, KAction::Autoloading);

    settings->setValue("kstopwatch/shortcut", seq.toString());
}

void MainWindow::slotTogglePause() {
    if(timer->isActive()) {
        slotPause();
    }
    else slotResume();
}
